@@ -16,6 +16,8 @@ import socket
 import sys
 import time
 
+from waitress.buffers import ReadOnlyFileBasedBuffer
+
 from waitress.compat import (
     tobytes,
     Queue,
@@ -147,7 +149,7 @@ class Task(object):
     status = '200 OK'
     wrote_header = False
     start_time = 0
-    content_length = -1
+    content_length = None
     content_bytes_written = 0
     logged_write_excess = False
     complete = False
@@ -208,7 +210,7 @@ class Task(object):
             # replace with properly capitalized version
             response_headers[i] = (headername, headerval)
 
-        if content_length_header is None and self.content_length != -1:
+        if content_length_header is None and self.content_length is not None:
             content_length_header = str(self.content_length)
             self.response_headers.append(
                 ('Content-Length', content_length_header)
@@ -258,6 +260,11 @@ class Task(object):
         res = '%s\r\n\r\n' % '\r\n'.join(lines)
         return tobytes(res)
 
+    def remove_content_length_header(self):
+        for i, (header_name, header_value) in enumerate(self.response_headers):
+            if header_name.lower() == 'content-length':
+                del self.response_headers[i]
+
     def start(self):
         self.start_time = time.time()
 
@@ -284,7 +291,7 @@ class Task(object):
                 # use chunked encoding response
                 towrite = tobytes(hex(len(data))[2:].upper()) + b'\r\n'
                 towrite += data + b'\r\n'
-            elif cl != -1:
+            elif cl is not None:
                 towrite = data[:cl-self.content_bytes_written]
                 self.content_bytes_written += len(towrite)
                 if towrite != data and not self.logged_write_excess:
@@ -293,7 +300,7 @@ class Task(object):
                         'bytes specified by Content-Length header (%s)' % cl)
                     self.logged_write_excess = True
             if towrite:
-                self.channel.write_soon(towrite)
+                channel.write_soon(towrite)
 
 class ErrorTask(Task):
     """ An error task produces an error response """
@@ -313,8 +320,7 @@ class ErrorTask(Task):
         self.write(tobytes(body))
 
 class WSGITask(Task):
-    """A WSGI task accepts a request and writes to a channel.
-
+    """A WSGI task produces a response from a WSGI application.
     """
 
     environ = None
@@ -374,8 +380,18 @@ class WSGITask(Task):
         app_iter = self.channel.server.application(env, start_response)
 
         try:
-            # By iterating manually at this point, we execute task.write()
-            # multiple times, allowing partial data to be sent.
+            if app_iter.__class__ is ReadOnlyFileBasedBuffer:
+                cl = self.content_length
+                size = app_iter.prepare(cl)
+                if size:
+                    if cl != size:
+                        if cl is not None:
+                            self.remove_content_length_header()
+                        self.content_length = size
+                    self.write(b'') # generate headers
+                    self.channel.write_soon(app_iter)
+                    return
+
             first_chunk_len = None
             for chunk in app_iter:
                 if first_chunk_len is None:
@@ -384,7 +400,7 @@ class WSGITask(Task):
                     # start_response may not have been called until first
                     # iteration as per PEP, so we must reinterrogate
                     # self.content_length here
-                    if self.content_length == -1:
+                    if self.content_length is None:
                         app_iter_len = None
                         if hasattr(app_iter, '__len__'):
                             app_iter_len = len(app_iter)
@@ -396,14 +412,14 @@ class WSGITask(Task):
                     self.write(chunk)
 
             cl = self.content_length
-            if cl != -1:
+            if cl is not None:
                 if self.content_bytes_written != cl:
                     # close the connection so the client isn't sitting around
                     # waiting for more data when there are too few bytes
                     # to service content-length
                     self.close_on_finish = True
                     self.logger.warning(
-                        'appplication returned too few bytes (%s) '
+                        'application returned too few bytes (%s) '
                         'for specified Content-Length (%s) via app_iter' % (
                             self.content_bytes_written, cl),
                         )
@@ -453,6 +469,8 @@ class WSGITask(Task):
         environ['wsgi.multiprocess'] = False
         environ['wsgi.run_once'] = False
         environ['wsgi.input'] = request.get_body_stream()
+        environ['wsgi.file_wrapper'] = ReadOnlyFileBasedBuffer
 
         self.environ = environ
         return environ
+
