@@ -11,6 +11,12 @@ def app(environ, start_response):
     return [body]
 
 if __name__ == '__main__':
+    import logging
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
+    h = NullHandler()
+    logging.getLogger('waitress').addHandler(h)
     from waitress import serve
     serve(app, port=61523, _quiet=True)
     
