@@ -19,16 +19,12 @@ class SubprocessTests(object):
     host = 'localhost'
     def start_subprocess(self, cmd):
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        cwd = os.getcwd()
-        os.chdir(dn(dn(here)))
         self.proc = subprocess.Popen(cmd)
-        os.chdir(cwd)
         time.sleep(.2)
         if self.proc.returncode is not None: # pragma: no cover
             raise RuntimeError('%s didnt start' % str(cmd))
 
     def stop_subprocess(self):
-        time.sleep(.2)
         if self.proc.returncode is None:
             #self.conn.close()
             self.proc.terminate()
@@ -40,6 +36,37 @@ class SubprocessTests(object):
         self.assertEqual(r, tobytes(reason))
         self.assertEqual(v, tobytes(version))
 
+class SleepyThreadTests(SubprocessTests, unittest.TestCase):
+    # test that sleepy thread doesnt block other requests
+    def setUp(self):
+        echo = os.path.join(here, 'fixtureapps', 'sleepy.py')
+        self.start_subprocess([self.exe, echo])
+
+    def tearDown(self):
+        self.stop_subprocess()
+
+    def test_it(self):
+        getline = os.path.join(here, 'fixtureapps', 'getline.py')
+        cmds = (
+            [self.exe, getline, 'http://127.0.0.1:%s/sleepy' % self.port],
+            [self.exe, getline, 'http://127.0.0.1:%s/' % self.port]
+            )
+        r, w = os.pipe()
+        procs = []
+        for cmd in cmds:
+            procs.append(subprocess.Popen(cmd, stdout=w))
+        time.sleep(3)
+        for proc in procs:
+            if proc.returncode is not None: # pragma: no cover
+                proc.terminate()
+        # the notsleepy response should always be first returned (it sleeps
+        # for 2 seconds, then returns; the notsleepy response should be 
+        # processed in the meantime)
+        result = os.read(r, 10000)
+        os.close(r)
+        os.close(w)
+        self.assertEqual(result, b'notsleepy returnedsleepy returned')
+
 class EchoTests(SubprocessTests, unittest.TestCase):
     def setUp(self):
         echo = os.path.join(here, 'fixtureapps', 'echo.py')
