@@ -169,6 +169,16 @@ foo: bar"""
         self.parser.parse_header(data)
         self.assertEqual(self.parser.connection_close, True)
 
+    def test__close_with_body_rcv(self):
+        body_rcv = DummyBodyStream()
+        self.parser.body_rcv = body_rcv
+        self.parser._close()
+        self.assertTrue(body_rcv.closed)
+
+    def test__close_with_no_body_rcv(self):
+        self.parser.body_rcv = None
+        self.parser._close() # doesn't raise
+
 class Test_split_uri(unittest.TestCase):
     def _callFUT(self, uri):
         from waitress.parser import split_uri
@@ -351,3 +361,10 @@ Hello.
 class DummyBodyStream(object):
     def getfile(self):
         return self
+
+    def getbuf(self):
+        return self
+
+    def _close(self):
+        self.closed = True
+        
